{% load i18n %}
<div class="dff-uploader">
    <div
        class="dff-container"
        data-translations="{{ translations }}"
    >
        {% include "django/forms/widgets/input.html" %}
    </div>
</div>
